package cat.hack3.codingtests.marsrover;

import cat.hack3.codingtests.marsrover.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.cartography.Direction;
import cat.hack3.codingtests.marsrover.cartography.MarsMap;

import java.util.logging.Logger;

public class MarsRover implements RotableRiderRover{
    Logger logger = Logger.getLogger(this.getClass().getName());

    private final MarsMap marsMap;
    private Direction currentDirection;

    public MarsRover(MarsMap marsMap, Direction startingDirection) {
        this.marsMap = marsMap;
        currentDirection = startingDirection;
    }

    @Override
    public void moveTowards(Direction direction) {
        marsMap.updatePositionTowards(direction);
    }

    @Override
    public void rotateTowards(Rotation rotation) {
        var newDirection = switch (rotation) {
            case LEFT -> currentDirection.getNextDirectionRotatingMinus90Degrees();
            case RIGHT -> currentDirection.getNextDirectionRotating90Degrees();
        };
        reportNewDirection(rotation, newDirection);
        currentDirection = newDirection;
    }

    private void reportNewDirection(Rotation rotation, Direction newDirection) {
        logger.info(String.format("Rotated towards %s, now the direction is %s",
                rotation, newDirection));
    }

    @Override
    public Coordinates getCurrentCoordinates() {
        return marsMap.getCurrentPosition();
    }

    @Override
    public Direction getCurrentDirection() {
        return currentDirection;
    }
}
